package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.AngleMarkMapper;
import digital.laboratory.platform.reagent.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service

public class AngleMarkServiceImpl extends ServiceImpl<AngleMarkMapper, AngleMark> implements AngleMarkService {

    @Autowired
    private ReviewAndApproveService reviewAndApproveService;
    @Autowired
    private PurchaseListService purchaseListService;
    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;
    @Autowired
    private ComplianceCheckService complianceCheckService;
    @Autowired
    private PeriodVerificationImplementationService periodVerificationImplementationService;
    @Autowired
    private DeliveryRegistrationFormService deliveryRegistrationFormService;
    @Autowired
    private PurchaseCatalogueService purchaseCatalogueService;
    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;
    @Autowired
    private WarehousingRecordFormService warehousingRecordFormService;
    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;
    @Autowired
    private CentralizedRequestService centralizedRequestService;


    @Override
    public AngleMark getMark(DLPUser dlpUser) {

        AngleMark angleMark = new AngleMark();

        ReviewAndApprove reviewAndApproveList = reviewAndApproveService.getReviewAndApproveList(dlpUser);
        //审核审批角标
        angleMark.setReview(reviewAndApproveList.getCentralizedRequestVOList().size() +
                reviewAndApproveList.getCheckScheduleVOList().size() +
                reviewAndApproveList.getEvaluationFormVOList().size() +
                reviewAndApproveList.getDecentralizedRequestVOList().size() +
                reviewAndApproveList.getInstructionBookList().size() +
                reviewAndApproveList.getPurchaseCatalogueVOList().size() +
                reviewAndApproveList.getAcceptanceRecordFormVOList().size() +
                reviewAndApproveList.getPeriodVerificationImplementationVOList().size() +
                reviewAndApproveList.getStandardMaterialApprovalFormList().size() +
                reviewAndApproveList.getComplianceCheckVOList().size() +
                reviewAndApproveList.getPurchasingPlanVOList().size());

        //验收任务角标
        if (dlpUser.getPermissions().contains("reagent_acceptance_record_form_add")) {
            angleMark.setCheckAccept(acceptanceRecordFormService.list(Wrappers.<AcceptanceRecordForm>query()
                    .eq("status", 0).or()
                    .eq("status", -2).or()
                    .eq("status", -3).or()
                    .eq("status", -4)).size());
        }
        //采购目录角标
        if (dlpUser.getPermissions().contains("reagent_purchase_catalogue_commit")) {
            angleMark.setGatherBuyCatalog(purchaseCatalogueService.list(Wrappers.<PurchaseCatalogue>query()
                    .eq("status", 3).or()
                    .eq("status", -3)).size());
        }
        //采购清单角标
        if (dlpUser.getPermissions().contains("reagent_purchase_list_commit"))
            angleMark.setPurchaseList(purchaseListService.list(Wrappers.<PurchaseList>query()
                    .eq("status", 0)).size());

        //采购入库任务角标
        if (dlpUser.getPermissions().contains("reagent_warehousing_record_form_add")) {
            angleMark.setPurchaseWarehousingMark(warehousingRecordFormService.list(Wrappers.<WarehousingRecordForm>query()
                    .eq("status", 0).or()
                    .eq("status", 1)).size());
        }
        //标准储备溶液入库角标
        if (dlpUser.getPermissions().contains("reagent_standard_reserve_solution_warehousing")) {
            angleMark.setStorageOfSolutionMark(standardReserveSolutionService.list(Wrappers.<StandardReserveSolution>query()
                    .eq("status", 0)).size());
        }
        //标准物质归还角标
        if (dlpUser.getPermissions().contains("reagent_standard_material_application_return")) {
            angleMark.setReturnMark(standardMaterialApplicationService.list(Wrappers.<StandardMaterialApplication>query()
                    .eq("status", 0)).size());
        }
        //符合性检查角标
        if (dlpUser.getPermissions().contains("reagent_compliance_check_commit")) {
            angleMark.setConcordCheck(complianceCheckService.list(Wrappers.<ComplianceCheck>query()
                    .eq("status", 0).or()
                    .eq("status", -2).or()
                    .eq("status", -3).or()
                    .eq("status", -1)).size());
        }
        //出库任务角标
        if (dlpUser.getPermissions().contains("reagent_delivery_registration_form_commit")) {
            angleMark.setOutboundManagement(deliveryRegistrationFormService.list(Wrappers.<DeliveryRegistrationForm>query()
                    .eq("status", 0)).size());
        }
        //入库任务角标
        angleMark.setWarehouse(angleMark.getReturnMark() + angleMark.getStorageOfSolutionMark() + angleMark.getPurchaseWarehousingMark());

        //期间核查任务角标
        if (dlpUser.getPermissions().contains("reagent_period_verification_implementation_commit")) {
        angleMark.setCheckPlanList(periodVerificationImplementationService.list(Wrappers.<PeriodVerificationImplementation>query()
                .eq("commit_status", 0).or()
                .eq("commit_status", -2)).size());}

        if (dlpUser.getPermissions().contains("reagent_purchasing_plan_commit")) {
            angleMark.setProcurementApplyList(centralizedRequestService.list(Wrappers.<CentralizedRequest>query()
                    .eq("status", 6)).size());}

        return angleMark;

    }
}
