package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PeriodVerificationImplementationDTO;
import digital.laboratory.platform.reagent.entity.PeriodVerificationImplementation;
import digital.laboratory.platform.reagent.entity.PeriodVerificationPlan;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.mapper.PeriodVerificationImplementationMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.InstructionBookVO;
import digital.laboratory.platform.reagent.vo.PeriodVerificationImplementationVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * (标准物质期间核查实施情况及结果记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查实施情况及结果记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class PeriodVerificationImplementationServiceImpl extends ServiceImpl<PeriodVerificationImplementationMapper, PeriodVerificationImplementation> implements PeriodVerificationImplementationService {
    @Autowired
    private PeriodVerificationPlanService periodVerificationPlanService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired

    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private InstructionBookService instructionBookService;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Override//分页查询
    public IPage<PeriodVerificationImplementationVO> getPeriodVerificationImplementationVOPage(IPage<PeriodVerificationImplementation> page, QueryWrapper<PeriodVerificationImplementation> qw) {

        IPage<PeriodVerificationImplementationVO> periodVerificationImplementationVOPage = baseMapper.getPeriodVerificationImplementationVOPage(page, qw);

        List<PeriodVerificationImplementationVO> records = periodVerificationImplementationVOPage.getRecords();

        for (PeriodVerificationImplementationVO record : records) {

            InstructionBookVO vo = instructionBookService.getVO(referenceMaterialService.getById(record.getReferenceMaterialId()).getReagentConsumableId());

            if (vo!=null){
                if (vo.isAuditResultOfTechnical()){

                record.setInstructionBook("有");}
            }
        }
        return periodVerificationImplementationVOPage;
    }

    @Override//通过ID查询
    public PeriodVerificationImplementationVO getPeriodVerificationImplementationVO(String periodVerificationImplementationId) {

        PeriodVerificationImplementationVO periodVerificationImplementationVO = baseMapper.getPeriodVerificationImplementationVO(periodVerificationImplementationId);

        return periodVerificationImplementationVO;
    }


    @Override//新增期间核查结果录入表
    public PeriodVerificationImplementation addById(PeriodVerificationPlan periodVerificationPlan) {

        PeriodVerificationImplementation periodVerificationImplementation = new PeriodVerificationImplementation();

        periodVerificationImplementation.setId(IdWorker.get32UUID().toUpperCase());
        //赋值信息
        periodVerificationImplementation.setDeviationAndUncertainty(periodVerificationPlan.getDeviationAndUncertainty());
        periodVerificationImplementation.setReferenceMaterialId(periodVerificationPlan.getReferenceMaterialId());
        periodVerificationImplementation.setReferenceMaterialNumber(periodVerificationPlan.getReferenceMaterialNumber());
        periodVerificationImplementation.setPeriodVerificationPlanId(periodVerificationPlan.getId());
        periodVerificationImplementation.setCauseOfDissatisfactionResult(false);



        LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();
        referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getId, periodVerificationPlan.getReferenceMaterialId());
        ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(referenceMaterialLambdaQueryWrapper);

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());
        //赋值信息
        periodVerificationImplementation.setReferenceMaterialName(reagentConsumables.getReagentConsumableName());
        periodVerificationImplementation.setStandardValueOrPurity(reagentConsumables.getStandardValueOrPurity());
        periodVerificationImplementation.setVerificationBasis(periodVerificationPlan.getVerificationBasis());
        periodVerificationImplementation.setCommitStatus(0);

        if (this.save(periodVerificationImplementation)) {
            return periodVerificationImplementation;
        } else throw new RuntimeException(String.format("保存失败"));

    }

    @Override//录入期间核查结果记录表
    public PeriodVerificationImplementation editById(PeriodVerificationImplementationDTO periodVerificationImplementationDTO, DLPUser dlpUser) {

        PeriodVerificationImplementation byId = this.getById(periodVerificationImplementationDTO.getPeriodVerificationImplementationId());

        BeanUtils.copyProperties(periodVerificationImplementationDTO, byId);

        byId.setInspectorId(dlpUser.getId());

        if (periodVerificationImplementationDTO.getCauseOfDissatisfaction() != null) {

            byId.setOpinionOfInspector(false);

        } else byId.setOpinionOfInspector(true);

        ReferenceMaterial byId1 = referenceMaterialService.getById(byId.getReferenceMaterialId());

        ReagentConsumables byId2 = reagentConsumablesService.getById(byId1.getReagentConsumableId());

        byId2.setDeviationOrUncertainty(periodVerificationImplementationDTO.getDeviationAndUncertainty());

        reagentConsumablesService.updateById(byId2);

        if (this.updateById(byId)) {

            return byId;
        } else throw new RuntimeException(String.format("保存失败"));

    }

    @Override//提交期间核查结果记录表
    public PeriodVerificationImplementation commitById(PeriodVerificationImplementationDTO periodVerificationImplementationDTO, DLPUser dlpUser) {

        PeriodVerificationImplementation periodVerificationImplementation = this.editById(periodVerificationImplementationDTO, dlpUser);

        periodVerificationImplementation.setCommitTime(LocalDateTime.now());

        String result = periodVerificationImplementationDTO.getResult();

        periodVerificationImplementation.setCheckingTime(LocalDate.now());

        if (periodVerificationImplementation.getCommitStatus()==-2){

            periodVerificationImplementation.setAuditOpinionOfTechnical("");
            periodVerificationImplementation.setAuditTimeOfTechnical(null);
            periodVerificationImplementation.setAuditResultOfTechnical(false);
            periodVerificationImplementation.setTechnicalDirectorId("");

        }

        periodVerificationImplementation.setCommitStatus(1);

        if (this.updateById(periodVerificationImplementation)) {

            return periodVerificationImplementation;

        } else return null;
    }

    @Override//审核期间核查结果记录表
    public PeriodVerificationImplementation auditById(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        PeriodVerificationImplementation byId = this.getById(auditAndApproveDTO.getUuId());

        byId.setAuditResultOfTechnical(auditAndApproveDTO.getAuditResult());
        byId.setAuditOpinionOfTechnical(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditTimeOfTechnical(LocalDate.now());
        byId.setTechnicalDirectorId(dlpUser.getId());

        if (auditAndApproveDTO.getAuditResult() == true) {

            PeriodVerificationPlan periodVerificationPlan = periodVerificationPlanService.getById(byId.getPeriodVerificationPlanId());

            periodVerificationPlan.setVerificationResult(byId.getResult());

            periodVerificationPlan.setImplementationDate(byId.getCheckingTime());

            periodVerificationPlan.setInspectorId(byId.getInspectorId());

            periodVerificationPlan.setDeviationAndUncertainty(byId.getDeviationAndUncertainty());

            periodVerificationPlan.setDateOfNextCheck(byId.getCheckingTime().plusMonths(periodVerificationPlan.getPlannedVerificationCycle()));

            periodVerificationPlanService.updateById(periodVerificationPlan);

            byId.setCommitStatus(6);
        } else byId.setCommitStatus(-2);

        if (this.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override
    public void periodVerificationImplementationTablePDF(PeriodVerificationImplementationVO periodVerificationImplementationVO, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("periodVerificationImplementationTablePDF.................");

        String id = periodVerificationImplementationVO.getId();

        String applyFileName = "标准物质期间核查实施情况及结果记录表" + id;
        //-----------------------------
        // 生成 word 版本的 验收记录表

        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        ossFile.fileGet("template" + "/" + "标准物质期间核查实施情况及结果记录表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);

        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        builder.bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("year", year);
                    put("referenceMaterialName", periodVerificationImplementationVO.getReferenceMaterialName());
                    put("referenceMaterialNumber", periodVerificationImplementationVO.getReferenceMaterialNumber());
                    put("standardValueOrPurity", periodVerificationImplementationVO.getStandardValueOrPurity());
                    put("deviationAndUncertainty", periodVerificationImplementationVO.getDeviationAndUncertainty());
                    put("inspectorName", periodVerificationImplementationVO.getInspectorName());
                    put("verificationMethod", periodVerificationImplementationVO.getVerificationMethod());
                    put("implementationAndResults", periodVerificationImplementationVO.getImplementationAndResults());
                    put("causeOfDissatisfactionResult", periodVerificationImplementationVO.isCauseOfDissatisfactionResult());
                    put("causeOfDissatisfaction", periodVerificationImplementationVO.getCauseOfDissatisfaction());
                    put("auditOpinionOfTechnical", periodVerificationImplementationVO.getAuditOpinionOfTechnical());
                    put("technicalDirectorName", periodVerificationImplementationVO.getTechnicalDirectorName());
                    put("remarks", periodVerificationImplementationVO.getRemarks());
                    put("checkingTime", LocalDateTimeUtil.format(periodVerificationImplementationVO.getCheckingTime(), "yyyy年MM月dd日"));
                    put("auditTimeOfTechnical", LocalDateTimeUtil.format(periodVerificationImplementationVO.getAuditTimeOfTechnical(), "yyyy年MM月dd日"));
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();

        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();

        ossFile.fileSave("document" + "/" + "periodVerificationImplementation" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();

        System.out.println(String.format("转换为 PDF 结束"));

    }
}
