package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.feign.RemoteTemplate2htmlService;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.ReagentConsumableInventoryDTO;
import digital.laboratory.platform.reagent.utils.IntegerUtils;
import digital.laboratory.platform.reagent.utils.PageUtils;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.ReagentConsumableInventoryMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import digital.laboratory.platform.sys.entity.CabinetCell;
import digital.laboratory.platform.sys.entity.CellAndStoreSupplyData;
import digital.laboratory.platform.sys.feign.RemoteCabinetService;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 试剂耗材库存服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 服务实现类
 */
@Service
@SuppressWarnings("all")
public class ReagentConsumableInventoryServiceImpl extends ServiceImpl<ReagentConsumableInventoryMapper, ReagentConsumableInventory> implements ReagentConsumableInventoryService {
    @Autowired
    private BatchDetailsService batchDetailsService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private RemoteTemplate2htmlService remoteTemplate2htmlService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private FixedValueReportService fixedValueReportService;

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private InstructionBookService instructionBookService;

    @Autowired
    private RemoteCabinetService remoteCabinetService;

    @Autowired
    private ReagentConsumableStashService reagentConsumableStashService;

    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;


    @Autowired
    private OssFile ossFile;

    @Override//标准物质标准物质管理列表
    public IPage<ReagentConsumableInventoryVO> getReagentConsumableInventoryRMVOList(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw, Integer warning) {


        if (warning != null) {

            if (warning == 2) {

                List<ReagentConsumableInventoryVO> reagentConsumableInventoryVOList = new ArrayList<>();

                List<BatchDetails> list = batchDetailsService.list(Wrappers.<BatchDetails>query()
                        .eq("warning_information", "即将过期").or()
                        .eq("warning_information", "已过期").or()
                        .eq("warning_information", "即将到达存储期限").or()
                        .eq("warning_information", "已超过存储期限"));

                if (list.size() != 0) {
                    for (BatchDetails batchDetails : list) {

                        ReagentConsumableInventory byId = this.getById(batchDetails.getReagentConsumableInventoryId());
                        ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();
                        BeanUtils.copyProperties(byId, reagentConsumableInventoryVO);
                        reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId()));
                        if (!reagentConsumableInventoryVOList.contains(reagentConsumableInventoryVO)) {
                            if (reagentConsumableInventoryVO.getCategory().equals("标准物质")) {
                                reagentConsumableInventoryVOList.add(reagentConsumableInventoryVO);
                            }
                        }

                    }
                }
                PageUtils pageUtils = new PageUtils();

                return pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOList);
            }
        }


        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryVOPage = baseMapper.getReagentConsumableInventoryVOPage(page, qw);

        List<ReagentConsumableInventoryVO> records = reagentConsumableInventoryVOPage.getRecords();

        for (ReagentConsumableInventoryVO reagentConsumableInventoryVO : records) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId());
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);
        }
        return reagentConsumableInventoryVOPage;
    }

    @Override//试剂耗材管理列表
    public IPage<ReagentConsumableInventoryVO> getReagentConsumableInventoryREVOList(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw, Integer warning) {

        if (warning != null) {

            if (warning == 2) {

                List<ReagentConsumableInventoryVO> reagentConsumableInventoryVOList = new ArrayList<>();

                List<BatchDetails> list = batchDetailsService.list(Wrappers.<BatchDetails>query()
                        .eq("warning_information", "即将过期").or()
                        .eq("warning_information", "已过期").or()
                        .eq("warning_information", "即将到达存储期限").or()
                        .eq("warning_information", "已超过存储期限"));
                //查找具有到期批次物品的试剂耗材类信息
                if (list.size() != 0) {
                    for (BatchDetails batchDetails : list) {

                        ReagentConsumableInventory byId = this.getById(batchDetails.getReagentConsumableInventoryId());
                        ReagentConsumableInventoryVO reagentConsumableInventoryVO = new ReagentConsumableInventoryVO();
                        BeanUtils.copyProperties(byId, reagentConsumableInventoryVO);
                        reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId()));

                        if (!reagentConsumableInventoryVOList.contains(reagentConsumableInventoryVO)) {
                            if (reagentConsumableInventoryVO.getCategory().equals("试剂") || reagentConsumableInventoryVO.getCategory().equals("耗材")) {
                                reagentConsumableInventoryVOList.add(reagentConsumableInventoryVO);
                            }
                        }

                    }
                }
                PageUtils pageUtils = new PageUtils();

                return pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryVOList);
            }
        }
        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryVOPage = baseMapper.getReagentConsumableInventoryVOPage(page, qw);

        List<ReagentConsumableInventoryVO> records = reagentConsumableInventoryVOPage.getRecords();

        for (ReagentConsumableInventoryVO reagentConsumableInventoryVO : records) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId());
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);
        }
        return reagentConsumableInventoryVOPage;
    }


    @Override//入库增加库存
    public ReagentConsumableInventory addById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = this.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() + quantity);


        if (this.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存添加失败"));
    }

    @Override//领用减少库存
    public ReagentConsumableInventory reduceById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = this.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() - quantity);

        if (one.getTotalQuantity() <= one.getWarningValue()) {

            one.setWarningInformation("库存不足");
        }

        if (this.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存减少失败"));
    }


    @Override//分页查询标准物质
    public IPage<ReferenceMaterialFullVO> getAllList(Page page, QueryWrapper<ReferenceMaterial> qw, Integer status, String number) {
        //如果扫码

        if (StrUtil.isNotBlank(number)) {

            ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(Wrappers.<ReferenceMaterial>query().eq("number", number));

            if (referenceMaterial != null) {

                ReagentConsumableInventoryFullVO byCode = this.getByCode(referenceMaterial.getId());

                List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOS = new ArrayList<>();

                reagentConsumableInventoryFullVOS.add(byCode);

                PageUtils pageUtils = new PageUtils();

                Page pages = pageUtils.getPages(page.getCurrent(), page.getSize(), reagentConsumableInventoryFullVOS);

                return pages;

            }
        }

        IPage<ReferenceMaterialFullVO> referenceMaterialPage = baseMapper.getReferenceMaterialPage(page, qw);

//        List<ReferenceMaterialFullVO> referenceMaterialPageRecords = referenceMaterialPage.getRecords();
//
//        LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionQueryWrapper = new LambdaQueryWrapper<>();
//
//        for (ReferenceMaterialFullVO referenceMaterialPageRecord : referenceMaterialPageRecords) {
//            standardReserveSolutionQueryWrapper.eq(StandardReserveSolution::getReferenceMaterialId,referenceMaterialPageRecord.getReferenceMaterialId());
//            StandardReserveSolution standardReserveSolution = standardReserveSolutionService.getOne(standardReserveSolutionQueryWrapper);
//
//            if (standardReserveSolution != null) {
//
//
//                referenceMaterialPageRecord.setConfigurationConcentration(standardReserveSolution.getConfigurationConcentration());
//                referenceMaterialPageRecord.setConfigurationDate(standardReserveSolution.getConfigurationDate());
//                referenceMaterialPageRecord.setSolutionNumbering(standardReserveSolution.getSolutionNumbering());
//                referenceMaterialPageRecord.setValidityPeriod(standardReserveSolution.getValidityPeriod());
//            }
//        }

        return referenceMaterialPage;
    }


    @Override//通过ID，获取标准物质所有信息
    public ReagentConsumableInventoryFullVO getByCode(String id) {

        ReferenceMaterial referenceMaterial = referenceMaterialService.getById(id);

        if (referenceMaterial == null) {
            throw new RuntimeException(String.format("未能查询到该ID对应的标准物质"));
        }
        ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

        ReagentConsumableInventory byId = this.getById(referenceMaterial.getReagentConsumableInventoryId());

        BeanUtils.copyProperties(byId, reagentConsumableInventoryFullVO);

        reagentConsumableInventoryFullVO.setNumber(referenceMaterial.getNumber());

        reagentConsumableInventoryFullVO.setBatchDetailsId(referenceMaterial.getBatchDetailsId());

        BatchDetails batchDetails = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());

        SupplierInformation supplierInformation = supplierInformationService.getById(batchDetails.getSupplierId());

        LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionLambdaQueryWrapper = new LambdaQueryWrapper<>();

        standardReserveSolutionLambdaQueryWrapper.eq(StandardReserveSolution::getReferenceId, referenceMaterial.getId());

        StandardReserveSolution one1 = standardReserveSolutionService.getOne(standardReserveSolutionLambdaQueryWrapper);

        reagentConsumableInventoryFullVO.setSupplierName(supplierInformation.getSupplierName());
        reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());
        reagentConsumableInventoryFullVO.setBatchNumber(batchDetails.getBatchNumber());
        reagentConsumableInventoryFullVO.setManufacturerId(batchDetails.getSupplierId());
        reagentConsumableInventoryFullVO.setPurchaseTime(batchDetails.getPurchaseTime());
        reagentConsumableInventoryFullVO.setReferenceMaterialId(referenceMaterial.getId());
        reagentConsumableInventoryFullVO.setReferenceMaterialStatus(referenceMaterial.getStatus());
        reagentConsumableInventoryFullVO.setFixedResult(referenceMaterial.getFixedResult());
        if (one1 != null) {
            reagentConsumableInventoryFullVO.setConfigurationConcentration(one1.getConfigurationConcentration());
            reagentConsumableInventoryFullVO.setConfigurationDate(one1.getConfigurationDate());
            reagentConsumableInventoryFullVO.setSolutionNumbering(referenceMaterial.getNumber());
            reagentConsumableInventoryFullVO.setValidityPeriod(one1.getValidityPeriod());
        }


        return reagentConsumableInventoryFullVO;
    }


    //分页查询试剂耗材
    @Override
    public IPage<ReagentConsumableInventoryFullVO> getAllRM(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw) {

        List<ReagentConsumableInventory> list = this.list(qw);

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetails> list1 = batchDetailsService.list(Wrappers.<BatchDetails>query()
                    .eq("reagent_consumable_inventory_id", reagentConsumableInventory.getReagentConsumableInventoryId())
                    .eq("service_status", 1));

            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                reagentConsumableInventoryFullVO.setSupplierName(supplierInformationService.getById(batchDetails.getSupplierId()).getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOS.add(reagentConsumableInventoryFullVO);
            }
        }
        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryFullVOS);

        return pages;

    }

    @Override
    public List<ReagentConsumableInventoryFullVO> getReagentConsumableInventoryFull(QueryWrapper<ReagentConsumableInventory> qw) {
        List<ReagentConsumableInventory> list = this.list(qw);
//        return list;

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();
        //找出对应的仓库信息
        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            batchDetailsLambdaQueryWrapper.eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                    .or().eq(BatchDetails::getServiceStatus, 1);

            List<BatchDetails> list1 = batchDetailsService.list(batchDetailsLambdaQueryWrapper);
            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }
        return reagentConsumableInventoryFullVOList;
    }


    @Override
    public void referenceMaterialTablePDF(List<ReferenceMaterialVO> voList, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("referenceMaterialTablePDF.................");

        int x = 1;

        for (ReferenceMaterialVO referenceMaterialVO : voList) {

            referenceMaterialVO.setI(x);
            x++;
        }
        //-----------------------------
        // 生成 word 版本的 采购目录
        String applyFileName = "标准物质一览表-" + id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "标准物质一览表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);
        builder.bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("year", year);
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "referenceMaterial" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }

    //试剂/耗材扫描物品编码进行持久化
    @Override
    public void setCode(String id, String code) {

        ReagentConsumables byId = reagentConsumablesService.getById(id);

        ReagentConsumableInventory one = this.getOne(Wrappers.<ReagentConsumableInventory>query()
                .eq("reagent_consumable_id", id));

        if (byId != null & one != null) {
            byId.setCode(code);
            one.setCode(code);

            this.updateById(one);
            reagentConsumablesService.updateById(byId);
        }

    }

    @Override
    public void setRMCode(String id, String code) {

        ReferenceMaterial referenceMaterial = referenceMaterialService.getById(id);

        if (referenceMaterial != null) {

            referenceMaterial.setCode(code);

            referenceMaterialService.updateById(referenceMaterial);
        }
    }

    @Override
//标准物质二维码打印
    public String buildCodeLabelContent(String id) {
        //录入二维码信息
        ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(id);

        referenceMaterialServiceById.setCode(referenceMaterialServiceById.getNumber());
        referenceMaterialService.updateById(referenceMaterialServiceById);

        if (referenceMaterialServiceById == null) {

            throw new RuntimeException(String.format("请先入库后再进行打印条码"));
        }
        //获取打印信息
        Map<String, Object> data = new HashMap<>();
        referenceMaterialService.updateById(referenceMaterialServiceById);
        ReagentConsumables byId = reagentConsumablesService.getById(referenceMaterialServiceById.getReagentConsumableId());
        String reagentConsumableName = byId.getReagentConsumableName();
        ReferenceMaterialVO referenceMaterial = new ReferenceMaterialVO();
        BatchDetails byId1 = batchDetailsService.getById(referenceMaterialServiceById.getBatchDetailsId());
        BeanUtils.copyProperties(referenceMaterialServiceById, referenceMaterial);
        //完善打印信息
        referenceMaterial.setReferenceMaterialName(byId.getReagentConsumableName());
        referenceMaterial.setTime("到期:" + byId1.getExpirationDate());
        data.put("referenceMaterial", referenceMaterial);
        String templateFileName = "标准物质标签模板.vm";

        return remoteTemplate2htmlService.getHtml(templateFileName, data);
    }

    @Override
    public String printSolutionTag(String id) {

        StandardReserveSolutionVO standardReserveSolutionVOById = standardReserveSolutionService.getStandardReserveSolutionVOById(id);
        Map<String, Object> data = new HashMap<>();
        data.put("standardReserveSolution", standardReserveSolutionVOById);
        String templateFileName = "标准储备溶液标签模板.vm";
        return remoteTemplate2htmlService.getHtml(templateFileName, data);
    }

    @Override
    public List<UserVO> getUserList() {

        return baseMapper.getUserList();
    }

    @Override
    @Transactional//盘点导入，需提供柜子系统数据
    public void inventoryEntry(List<ReagentConsumableInventoryDTO> reagentConsumableInventoryDTOList) {

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);

        IntegerUtils integerUtils = new IntegerUtils();
        for (ReagentConsumableInventoryDTO reagentConsumableInventoryDTO : reagentConsumableInventoryDTOList) {
            Integer copies = integerUtils.getInteger(reagentConsumableInventoryDTO.getPackagedCopies());
            Integer quantity = reagentConsumableInventoryDTO.getTotalQuantity() * copies;
            SupplierInformation supplier = supplierInformationService.getOne(Wrappers.<SupplierInformation>query().eq("supplier_name", reagentConsumableInventoryDTO.getSupplierName()));


            String location = "F" + reagentConsumableInventoryDTO.getRoomNo().charAt(0) + "-" + reagentConsumableInventoryDTO.getRoomNo() + "_试剂柜-" + reagentConsumableInventoryDTO
                    .getLocation().charAt(0) + "_" + reagentConsumableInventoryDTO.getLocation();

            ReagentConsumableInventory one = this.getOne(Wrappers.<ReagentConsumableInventory>query()
                    .eq("reagent_consumable_name", reagentConsumableInventoryDTO.getReagentConsumableName())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getBrand()), "brand", reagentConsumableInventoryDTO.getBrand())
                    .eq("category", reagentConsumableInventoryDTO.getCategory())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getSpecificationAndModel()), "specification_and_model", reagentConsumableInventoryDTO.getSpecificationAndModel())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getStandardValueOrPurity()), "standard_value_or_purity", reagentConsumableInventoryDTO.getStandardValueOrPurity())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getCasNumber()), "cas_number", reagentConsumableInventoryDTO.getCasNumber())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getSpecies()), "species", reagentConsumableInventoryDTO.getSpecies())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getEnglishName()), "english_name", reagentConsumableInventoryDTO.getEnglishName())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getPackagedCopies()), "packaged_copies", reagentConsumableInventoryDTO.getPackagedCopies())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getAlias()), "alias", reagentConsumableInventoryDTO.getAlias())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getPurityGrade()), "purity_grade", reagentConsumableInventoryDTO.getPurityGrade())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getDeviationOrUncertainty()), "deviation_or_uncertainty", reagentConsumableInventoryDTO.getDeviationOrUncertainty())
                    .eq(StrUtil.isNotBlank(reagentConsumableInventoryDTO.getMinimumUnit()), "minimum_unit", reagentConsumableInventoryDTO.getMinimumUnit())
            );
            BatchDetails batchDetails = new BatchDetails();
            batchDetails.setServiceStatus(1);
            batchDetails.setQuantity(quantity);
            batchDetails.setBatch(year + "-1");
            if (supplier == null) {
                SupplierInformation supplierInformation = new SupplierInformation();
                supplierInformation.setSupplierName(reagentConsumableInventoryDTO.getSupplierName());
                supplierInformation.setAcceptableCondition("合格");
                supplierInformation.setId(IdWorker.get32UUID());
                supplierInformationService.save(supplierInformation);
                batchDetails.setSupplierId(supplierInformation.getId());

            } else {
                batchDetails.setSupplierId(supplier.getId());
            }
            //如果没有生产日期/到期时间，就自动生成
            if (reagentConsumableInventoryDTO.getDateOfProduction() != null) {
                batchDetails.setDateOfProduction(reagentConsumableInventoryDTO.getDateOfProduction());
            } else {
                batchDetails.setDateOfProduction(LocalDate.now());
            }
            if (reagentConsumableInventoryDTO.getExpirationDate() != null) {
                batchDetails.setExpirationDate(reagentConsumableInventoryDTO.getExpirationDate());
            } else {
                batchDetails.setExpirationDate(LocalDate.now().plusYears(99));
            }
            R<CabinetCell> cabinetCellR = remoteCabinetService.queryCellByRoomNoAndLocation(reagentConsumableInventoryDTO.getRoomNo(), reagentConsumableInventoryDTO.getLocation());
            CabinetCell cabinetCell = cabinetCellR.getData();
            batchDetails.setLatticeId(cabinetCell.getId());
            batchDetails.setBoxId(cabinetCell.getCabinetId());
            batchDetails.setLocation(location);
            batchDetails.setPurchaseTime(LocalDateTime.now());
            batchDetails.setBatchNumber("/");
            batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
            batchDetails.setRoomNo(reagentConsumableInventoryDTO.getRoomNo());
            if (one != null) {
                List<BatchDetails> batchDetailsList = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_inventory_id", one.getReagentConsumableInventoryId()));
                if (batchDetailsList != null & batchDetailsList.size() > 0) {
                    int x = batchDetailsList.size() + 1;
                    batchDetails.setBatch("2023-" + x);
                    one.setTotalQuantity(one.getTotalQuantity() + quantity);
                    this.updateById(one);
                }
                batchDetails.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                ReagentConsumables reagentConsumables = reagentConsumablesService.getById(one.getReagentConsumableId());
                if (!one.getCategory().equals("标准物质")) {
                    ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getOne(Wrappers.<ReagentConsumableStash>query()
                            .eq("room_no", reagentConsumableInventoryDTO.getRoomNo())
                            .eq("reagent_consumable_id", reagentConsumables.getReagentConsumableId()));
                    if (reagentConsumableStash != null) {
                        reagentConsumableStash.setTotalQuantity(reagentConsumableStash.getTotalQuantity() + quantity);
                        batchDetails.setReagentConsumableStashId(reagentConsumableStash.getId());
                        reagentConsumableStashService.updateById(reagentConsumableStash);
                    } else {
                        ReagentConsumableStash consumableStash = new ReagentConsumableStash();
                        BeanUtils.copyProperties(reagentConsumableInventoryDTO, consumableStash);
                        consumableStash.setId(IdWorker.get32UUID().toUpperCase());
                        consumableStash.setTotalQuantity(quantity);
                        consumableStash.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                        batchDetails.setReagentConsumableStashId(consumableStash.getId());
                        reagentConsumableStashService.save(consumableStash);
                    }
                }
                if (reagentConsumableInventoryDTO.getCategory().equals("标准物质")) {
                    ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                    referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                    referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                    referenceMaterial.setLocation(location);
                    referenceMaterial.setStatus(0);
                    referenceMaterial.setLatticeId(cabinetCell.getId());
                    referenceMaterial.setBoxId(cabinetCell.getCabinetId());
                    String roomNo = reagentConsumableInventoryDTO.getRoomNo();
                    referenceMaterial.setNumber(reagentConsumableInventoryDTO.getAlias() + "-" + year + "-" + roomNo.charAt(0) + "-1");//自动生成编号
                    referenceMaterial.setCode(referenceMaterial.getNumber());
                    referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                    referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                    referenceMaterial.setReagentConsumableName(reagentConsumableInventoryDTO.getReagentConsumableName());
                    referenceMaterial.setMass(reagentConsumableInventoryDTO.getMass());
                    referenceMaterial.setReferenceMaterialUnit(reagentConsumableInventoryDTO.getReferenceMaterialUnit());
                    referenceMaterialService.save(referenceMaterial);
                    this.updateCabinet(cabinetCell.getId(), referenceMaterial.getId(), 1);
                }
                this.updateCabinet(cabinetCell.getId(), reagentConsumables.getReagentConsumableId(), 1);

            } else {
                ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();
                BeanUtils.copyProperties(reagentConsumableInventoryDTO, reagentConsumableInventory);
                reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());
                reagentConsumableInventory.setWarningValue(1);
                if (reagentConsumableInventoryDTO.getSpecificationAndModel().equals("/")) {
                    if (reagentConsumableInventoryDTO.getPackages() != null) {
                        reagentConsumableInventory.setSpecificationAndModel(reagentConsumableInventoryDTO.getPackages());
                    }
                } else {
                    if (reagentConsumableInventoryDTO.getPackages() != null) {
                        reagentConsumableInventory.setSpecificationAndModel(reagentConsumableInventoryDTO.getSpecificationAndModel() + "," + reagentConsumableInventoryDTO.getPackages());
                    }
                }
                if (reagentConsumableInventoryDTO.getPackages() != null) {
                    reagentConsumableInventory.setSpecificationAndModel(reagentConsumableInventory.getSpecificationAndModel() + "," + reagentConsumableInventoryDTO.getPackages());
                }
                batchDetails.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
                ReagentConsumables reagentConsumables = reagentConsumablesService.addReagentConsumables(reagentConsumableInventory);
                reagentConsumableInventory.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                reagentConsumableInventory.setTotalQuantity(quantity);
                this.save(reagentConsumableInventory);
                this.updateCabinet(cabinetCell.getId(), reagentConsumables.getReagentConsumableId(), 1);

                if (!reagentConsumableInventory.getCategory().equals("标准物质")) {
                    ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getOne(Wrappers.<ReagentConsumableStash>query()
                            .eq("room_no", reagentConsumableInventoryDTO.getRoomNo())
                            .eq("reagent_consumable_id", reagentConsumables.getReagentConsumableId()));
                    if (reagentConsumableStash != null) {
                        reagentConsumableStash.setTotalQuantity(reagentConsumableStash.getTotalQuantity() + quantity);
                        batchDetails.setReagentConsumableStashId(reagentConsumableStash.getId());
                        reagentConsumableStashService.updateById(reagentConsumableStash);
                    } else {
                        ReagentConsumableStash consumableStash = new ReagentConsumableStash();
                        BeanUtils.copyProperties(reagentConsumableInventoryDTO, consumableStash);
                        consumableStash.setId(IdWorker.get32UUID().toUpperCase());
                        consumableStash.setTotalQuantity(quantity);
                        consumableStash.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                        batchDetails.setReagentConsumableStashId(consumableStash.getId());
                        reagentConsumableStashService.save(consumableStash);
                    }
                }
                if (reagentConsumableInventoryDTO.getCategory().equals("标准物质")) {
                    ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                    referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                    referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
                    referenceMaterial.setLocation(location);
                    referenceMaterial.setStatus(0);
                    referenceMaterial.setLatticeId(cabinetCell.getId());
                    referenceMaterial.setBoxId(cabinetCell.getCabinetId());
                    referenceMaterial.setReagentConsumableName(reagentConsumableInventoryDTO.getReagentConsumableName());
                    String roomNo = reagentConsumableInventoryDTO.getRoomNo();
                    referenceMaterial.setNumber(reagentConsumableInventoryDTO.getAlias() + "-" + year + "-" + roomNo.charAt(0) + "-1");//自动生成编号
                    referenceMaterial.setCode(referenceMaterial.getNumber());
                    referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                    referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                    referenceMaterial.setMass(reagentConsumableInventoryDTO.getMass());
                    referenceMaterial.setReferenceMaterialUnit(reagentConsumableInventoryDTO.getReferenceMaterialUnit());
                    referenceMaterialService.save(referenceMaterial);
                    this.updateCabinet(cabinetCell.getId(), referenceMaterial.getId(), 1);
                }
                this.updateCabinet(cabinetCell.getId(), reagentConsumables.getReagentConsumableId(), 1);
            }
            batchDetailsService.save(batchDetails);
        }
    }

    @Override
    //更新柜子系统方法，柜子ID、物品ID、入库OR出库 (opCode 1 or -1)
    public void updateCabinet(String latticeId, String itemId, int opCode) {
        ArrayList<CellAndStoreSupplyData> cellAndStoreSupplyDataArrayList = new ArrayList<>();//柜子数组
        CellAndStoreSupplyData cellAndStoreSupplyData = new CellAndStoreSupplyData();
        cellAndStoreSupplyData.setCellStatus("1");
        cellAndStoreSupplyData.setCellId(latticeId);
        ArrayList<String> arrayList = new ArrayList<>();
        arrayList.add(itemId);
        cellAndStoreSupplyData.setStoreSupplyList(arrayList);
        cellAndStoreSupplyDataArrayList.add(cellAndStoreSupplyData);
        remoteCabinetService.takeOrSaveGoodsFromCabinet(cellAndStoreSupplyDataArrayList, opCode);
    }


    @Override
    public List<OrgFullVO> getOrgList() {

        List<OrgVO> orgList = baseMapper.getOrgList();
        ArrayList<OrgFullVO> orgFullVOS = new ArrayList<>();
        for (OrgVO orgVO : orgList) {
            if (orgVO.getOrgType() != null && orgVO.getOrgType().equals("province") && !(orgVO.getAddress().substring(0, 2).equals("陕西"))) {
                OrgFullVO orgFullVO = new OrgFullVO();
                orgFullVO.setUsername("P" + orgVO.getAreaCode());
                String[] roles = {"10010", "10015", "10020", "10130", "23020", "23030"};
                orgFullVO.setRoleIds(roles);
                orgFullVO.setPassword("123456");
                orgFullVO.setOrgId(orgVO.getOrgId());
                orgFullVO.setGender(1);
                orgFullVOS.add(orgFullVO);
            }

        }
        System.out.println("总个数" + orgFullVOS.size());
        return orgFullVOS;
    }
}
